# -*- coding: utf-8 -*-
__all__ = ["test01", "test02", "test03", "test04"]

test01 = "https://osu.box.com/shared/static/yis83dhuj1q7v2srw2b1hjnb66d1rw1h.csdf"
test02 = "https://osu.box.com/shared/static/0jsvs9bmme8vf6yn76bctkhfxm4vkcc8.csdf"
test03 = "https://osu.box.com/shared/static/g4tzjpbvh2o8ew9y5imkhzbpyrlkwf8l.csdf"
test04 = "https://osu.box.com/shared/static/1cmop0icxqcphdydwsk9evj7mh0qtzaq.csdf"
